//#include "object.as"

speedTrapManager speedLimiter();

funcdef void SPEEDTRAP_EVENT_CALLBACK(float, int, speedTrap);

void speedEventHandler(int trigger_type, string inst, string box, int nodeid)
{
	speedLimiter.speedEventHandler(trigger_type, inst, box, nodeid);
}

class speedTrapManager
{	
	array<speedTrap> instances;
	int free_trap;
	string lastInstance;
	
	speedTrapManager()
	{
		free_trap = 0;
		lastInstance = "";
	}
	
	void addSpeedTrap(string category, object obj1, object obj2, float minSpeed, float maxSpeed, SPEEDTRAP_EVENT_CALLBACK@ callback)
	{
		obj1.eventHandler = "speedEventHandler";
		obj1.spawn("speed-start|"+free_trap);

		obj2.eventHandler = "speedEventHandler";
		obj2.spawn("speed-end|"+free_trap);

		speedTrap tmp();
		tmp.category      = category;
		@tmp.startObject  = @obj1;
		@tmp.endObject    = @obj2;
		tmp.minSpeed      = minSpeed;
		tmp.maxSpeed      = maxSpeed;
		@tmp.callback     = @callback;
		tmp.distance      = obj1.position.distance(obj2.position);
		tmp.lastStartTime = -1.0;
		tmp.hidden        = false;
		
		instances.resize(free_trap+1);
		instances[free_trap] = tmp;
		++free_trap;
	}
	
	float kph(float kph)
	{
		return kph*1000/3600;
	}
	
	float mph(float mph)
	{
		return mph*1609.344f/3600;
	}
	
	void speedEventHandler(int trigger_type, string inst, string box, int nodeid)
	{
		// the inst string contains the information about the event
		string[] tmp = explode('|', inst, true);
		if( tmp.length() < 2 )
			return;
		int trapID = intval(tmp[1]);
		string type = tmp[0];
		speedTrap@ trap = @instances[trapID];
		
		if( type == "speed-start" )
		{
			trap.lastStartTime = game.getTime();
			lastInstance = "";
		}
		else if( type == "speed-end" )
		{
			if( trap.lastStartTime < 0.0f )
				return;
			
			if( inst == lastInstance )
				return;
			lastInstance = inst;
			
			float speed = trap.distance / (game.getTime() - trap.lastStartTime);

			int compare = 0;
			if( speed < trap.minSpeed ) 
				compare = -1;
			else if( speed > trap.maxSpeed )
				compare = 1;
			
			if( @trap.callback !is null )
				trap.callback(speed, compare, trap);
			
			trap.lastStartTime = -1.0f;
		}
	}
	
	void hide(string category)
	{
		for( int i = 0; i < free_trap; ++i )
		{
			if( (category == instances[i].category) and not instances[i].hidden )
			{
				instances[i].startObject.hide();
				instances[i].endObject.hide();
				instances[i].hidden = true;
			}
		}
	}
	
	void show(string category)
	{
		for( int i = 0; i < free_trap; ++i )
		{
			if( (category == instances[i].category) and instances[i].hidden )
			{
				instances[i].startObject.spawn();
				instances[i].endObject.spawn();
				instances[i].hidden = false;
			}
		}
	}
	
	// Now we define a few utility functions because we can't use the default functions as they differ between RoR versions
	
	
	// Returns the ASCII value of the first character of a string.
	// This function complements chr()
	uint8 ord(const string &in character) const
	{
		if( character.length()<=0 )
		{
			log("ERROR| utils::ord(\""+character+"\") --> Found an empty string, while expecting a one-character string");
			return 0;
		}
		return character[0];
	}

	// Returns a one-character string containing the character specified by ascii. 
	// This function complements ord().
	string chr(const uint8 &in ascValue) const
	{
		string str = " ";
		str[0] = ascValue;
		return str;
	}
	
	// Returns an array of strings, each of which is a substring of string formed by splitting it on boundaries formed by the one-character string delimiter.
	string[] explode(const string &in delimiter, const string &in str_in, const bool &in includeEmpty) const
	{
		// We only need the first character or the delimiter
		uint8 delimiter_ord = ord(delimiter);
		
		int max = str_in.length()-1;
		string[] arr_out = {""};
		int arr_counter = 0;
		
		for(int i=0; i<=max; i++)
		{
			if( str_in[i] == delimiter_ord )
			{
				if(includeEmpty || arr_out[arr_counter].length()!=0)
				{
					++arr_counter;
					if(i<max)
					{
						arr_out.resize(arr_counter+1);
						arr_out[arr_counter] = "";
					}
				}
			}
			else
				arr_out[arr_counter] += chr(str_in[i]);
		}
		if(!includeEmpty && arr_out[arr_counter].length()==0)
			arr_out.resize(arr_counter);
		return arr_out;
	}
	
	int intval(const string &in number_in) const
	{		
		int number_out = 0;
		int intvalue;
		int multiplier = 1;
		const uint8 zero_ord = ord("0");
			
		for( int i = number_in.length()-1 ; i >= 0 ; i-- )
		{
			intvalue = number_in[i]-zero_ord;
			if( intvalue>=0 )
			{
				number_out += intvalue*multiplier;
				multiplier = multiplier*10;
			}
		}
		return number_out;
	}
}

class speedTrap
{
	object@ startObject;
	object@ endObject;
	float minSpeed;
	float maxSpeed;
	SPEEDTRAP_EVENT_CALLBACK@ callback;
	float distance;
	float lastStartTime;
	string category;
	bool hidden;
}